import Vue from 'vue';

Vue.config.productionTip = false;

let getTouchInfo = (startX, startY, endX, endY) => {
    let dy = startY - endY;
    let dx = endX - startX;
    let result = { direction: false }; // no move
    if (Math.abs(dx) < 2 && Math.abs(dy) < 2) {
        return result;
    }
    let angle = Math.atan2(dy, dx) * 180 / Math.PI;
    if (angle >= -45 && angle < 45) {
        result.direction = 'right';
        result.offset = dx;
    } else if (angle >= 45 && angle < 135) {
        result.direction = 'up';
        result.offset = dy;
    } else if (angle >= -135 && angle < -45) {
        result.direction = 'down';
        result.offset = dy;
    } else if ((angle >= 135 && angle <= 180) || (angle >= -180 && angle < -135)) {
        result.direction = 'left';
        result.offset = dx;
    }
    return result;
};

/* eslint-disable no-new */
new Vue({
    el: '#app',
    components: {
        app: { template: '<div id="main"><slot></slot></div>' }
    },
    data: {
        isTopReady: false,
        isTopAdding: false,
        isBottomAdding: false,
        isMoved: false,
        isSaved: false,
        items: [{
            parentId: 0,
            id: 'g1',
            name: 'g1'
        }, {
            parentId: 0,
            id: 'g2',
            name: 'g2'
        }, {
            parentId: 'g1',
            id: '1-1',
            name: '1-1'
        }],
        parentId: 0, // 通过这个 parentId 提取列表项
        form: {
            id: null,
            parentId: 0, // 数据保存时的 parentId, setForm() 会自动设置
            name: ''
        }
    },
    computed: {
        isAdding () {
            return this.isTopAdding || this.isBottomAdding;
        },
        isEditing () {
            return this.form.id !== null;
        },
        displayItems () {
            let result = [];
            result = this.items.filter((item) => { return item.parentId === this.parentId; });
            return result;
        }
    },
    methods: {
        sectionTouch ($event, vnode) {
            this.isSaved = false;
        },
        sectionMoving ($event, vnode, touchInfo) {
            if (this.isAdding || this.isEditing) {
                return;
            }
            // data refresh
            this.isMoved = false;

            // logic
            let el = vnode.elm;
            let direction = touchInfo.direction;
            let offset = touchInfo.offset;
            let $ul = el.querySelector('ul');
            if (['up', 'down'].indexOf(direction) !== -1) {
                $ul.style.transform = 'translateY(' + (offset * -1) + 'px)';
            }
            if (direction === 'down') {
                let angle = 90 - (offset / 60 * -100);
                let opacity = 0.5 + (offset / 60 * -0.5);
                let $rotate = $ul.querySelector('li.top div.rotate');
                $rotate.style.transform = 'rotateX(' + Math.max(angle, 0) + 'deg)';
                $rotate.style.opacity = opacity;
                this.isTopReady = Math.abs(offset) >= 60;
            }
        },
        sectionMoved ($event, vnode, touchInfo) {
            if (this.isAdding || this.isEditing) {
                return;
            }
            // data refresh
            this.isMoved = true;

            // logic
            let el = vnode.elm;
            let $ul = el.querySelector('ul');
            let $rotate = $ul.querySelector('li.top div.rotate');
            if (!this.isTopReady) {
                $ul.style.transform = 'translateY(0px)';
                $rotate.style.transform = 'rotateX(90deg)';
            } else {
                this.isTopReady = false;
                this.isTopAdding = true;
                $ul.style.transform = null;
                $rotate.style.transform = null;
                this.setForm();
            }
        },
        sectionTap ($event, vnode) {
            $event.stopPropagation();
            if ($event.target.nodeName.toLowerCase() === 'input') {
                return;
            }
            if (this.isEditing) {
                this.reset();
                return;
            }
            // data refresh
            if (!this.isAdding) {
                this.isBottomAdding = true;
                this.setForm();
            } else {
                this.reset();
            }
        },
        itemTap ($event, vnode) {
            $event.stopPropagation();
            if (this.isAdding) {
                this.reset();
                return;
            }
            let id = vnode.data.key;
            if (this.isEditing && this.form.id !== id) {
                this.reset();
                return;
            }
            let info = this.items.find(item => { return item.id === id; });
            this.setForm(info);
        },
        setForm (info) {
            Object.assign(this.form, info || {});
            this.form.parentId = this.parentId;
            // 这里统一设置输入信息
            setTimeout(() => {
                let $input = this.$refs.input;
                if (this.form.id !== null) {
                    $input = this.$refs.input[0];
                }
                $input.focus();
            });
        },
        reset (savedata = true) {
            if (savedata) {
                let info = this.items.find(item => { return item.id === this.form.id; }) || { id: new Date().getTime('x') };
                let name = this.form.name.replace(/^\s|\s$/ig, '');
                Object.assign(info, this.form, { id: info.id, name });
                if (this.isTopAdding) {
                    this.items.unshift(info);
                } else if (this.isBottomAdding) {
                    this.items.push(info);
                }
                if (name.length === 0) {
                    setTimeout(() => {
                        this.items.splice(this.items.findIndex(item => { return item.id === info.id; }), 1);
                    });
                }
            }
            this.isTopAdding = false;
            this.isBottomAdding = false;
            this.isSaved = true;
            this.form.id = null;
            this.form.name = '';
        }
    },
    directives: {
        gesture: {
            inserted (el, binding, vnode) {
                let options = binding.value;
                let startPoint = null;
                let endPoint = null;
                let isMoving = false;
                el.ontouchstart = function (e) {
                    startPoint = e.touches[0];
                    if (typeof options.onTouch === 'function') {
                        options.onTouch(e, vnode);
                    }
                };
                el.ontouchmove = function (e) {
                    isMoving = true;
                    endPoint = e.touches[0];
                    let touchInfo = getTouchInfo(startPoint.pageX, startPoint.pageY, endPoint.pageX, endPoint.pageY);
                    if (typeof options.onMove === 'function') {
                        options.onMove(e, vnode, touchInfo);
                    }
                };
                el.ontouchend = el.ontouchcancel = function (e) {
                    endPoint = e.changedTouches[0];
                    let touchInfo = getTouchInfo(startPoint.pageX, startPoint.pageY, endPoint.pageX, endPoint.pageY);
                    if (isMoving && typeof options.onEnd === 'function') {
                        options.onEnd(e, vnode, touchInfo);
                    }
                    startPoint = null;
                    endPoint = null;
                    isMoving = false;
                };
                el.onmouseup = function (e) {
                    if (!isMoving && typeof options.onTap === 'function') {
                        options.onTap(e, vnode);
                    }
                };
            }
        }
    },
    mounted () {
        require('./index.less');
        console.log('apps/clear => mounted()');
    }
});
